@@ -745,19 +745,15 @@ var STRICT = 0;
 var WARN_ON_UNDEFINED_SYMBOLS = 1;
 
 // If set to 1, we will give a link-time error on any undefined symbols (see
-// WARN_ON_UNDEFINED_SYMBOLS). The default value is 1. To allow undefined
-// symbols at link time set this to 0, in which case if an undefined function is
-// called a runtime error will occur.  Any undefined symbols that are listed in
-// EXPORTED_FUNCTIONS will also be reported.
+// WARN_ON_UNDEFINED_SYMBOLS). To allow undefined symbols at link time set this
+// to 0, in which case if an undefined function is called a runtime error will
+// occur.  Any undefined symbols that are listed in EXPORTED_FUNCTIONS will also
+// be reported.
 var ERROR_ON_UNDEFINED_SYMBOLS = 1;
 
-// If set to 1, any -lfoo directives pointing to nonexistent library files will
-// issue a linker error.
-
-// The default value for this is currently 0, but will be transitioned to 1 in
-// the future. To keep relying on building with -s ERROR_ON_MISSING_LIBRARIES=0
-// setting, prefer to set that option explicitly in your build system.
-var ERROR_ON_MISSING_LIBRARIES = 0;
+// My default any -lfoo directives pointing to nonexistent library files will
+// issue a linker error.  Set to 0 to downgrade this to a warning.
+var ERROR_ON_MISSING_LIBRARIES = 1;
 
 // Specifies a list of Emscripten-provided JS libraries to link against.
 // (internal, use -lfoo or -lfoo.js to link to Emscripten system JS libraries)
@@ -1080,6 +1076,12 @@ var USE_ICU = 0;
 // 1 = use zlib from emscripten-ports
 var USE_ZLIB = 0;
 
+// 1 = use bzip2 from emscripten-ports
+var USE_BZIP2 = 0;
+
+// 1 = use libjpeg from emscripten-ports
+var USE_LIBJPEG = 0;
+
 // 1 = use libpng from emscripten-ports
 var USE_LIBPNG = 0;
 
@@ -1320,11 +1322,6 @@ var STATIC_BUMP = -1;
 // the total initial wasm table size.
 var WASM_TABLE_SIZE = 0;
 
-// if set to 1, then generated WASM files will contain a custom
-// "emscripten_metadata" section that contains information necessary
-// to execute the file without the accompanying JS file.
-var EMIT_EMSCRIPTEN_METADATA = 0;
-
 // Tracks whether we are building with errno support enabled. Set to 0
 // to disable compiling errno support in altogether. This saves a little
 // bit of generated code size in applications that do not care about
@@ -1388,6 +1385,16 @@ var DISABLE_DEPRECATED_FIND_EVENT_TARGET_BEHAVIOR = 0;
 // to explicitly choose to disable HTML minification altogether.
 var MINIFY_HTML = 1;
 
+// Indicates that the syscalls (which we see statically) indicate that they need full
+// filesystem support. Otherwise, when just a small subset are used, we can get away without
+// including the full filesystem - in particular, if open() is never used, then we don't
+// actually need to support operations on streams.
+var SYSCALLS_REQUIRE_FILESYSTEM = 1;
+
+// A list of feature flags to pass to each binaryen invocation (like wasm-opt, etc.). This
+// is received from wasm-emscripten-finalize, which reads it from the features section.
+var BINARYEN_FEATURES = [];
+
 // Legacy settings that have been removed, and the values they are now fixed to.
 // These can no longer be changed:
 // [OPTION_NAME, POSSIBLE_VALUES, ERROR_EXPLANATION], where POSSIBLE_VALUES is
