@@ -133,7 +133,7 @@ def get_native_optimizer():
       for cmake_generator in cmake_generators:
         # Delete CMakeCache.txt so that we can switch to a new CMake generator.
         shared.try_delete(os.path.join(build_path, 'CMakeCache.txt'))
-        proc = subprocess.Popen(['cmake', '-G', cmake_generator, '-DCMAKE_BUILD_TYPE='+cmake_build_type, shared.path_from_root('tools', 'optimizer')], cwd=build_path, stdin=log_output, stdout=log_output, stderr=log_output)
+        proc = subprocess.Popen(['cmake', '-G', cmake_generator, '-DCMAKE_BUILD_TYPE='+cmake_build_type, shared.path_from_root('tools', 'optimizer')], cwd=build_path, stdin=log_outpu, stdout=log_output, stderr=log_output)
         proc.communicate()
         if proc.returncode == 0:
           make = ['cmake', '--build', build_path]
@@ -175,9 +175,9 @@ def get_native_optimizer():
 
     use_cmake_to_configure = WINDOWS # Currently only Windows uses CMake to drive the optimizer build, but set this to True to use on other platforms as well.
     if use_cmake_to_configure:
-      return shared.Cache.get(name, create_optimizer_cmake, extension='exe')
+      return shared.Cache.get(name, create_optimizer_cmake)
     else:
-      return shared.Cache.get(name, create_optimizer, extension='exe')
+      return shared.Cache.get(name, create_optimizer)
 
   if NATIVE_OPTIMIZER == '1':
     return get_optimizer('optimizer.exe', [])
@@ -325,8 +325,6 @@ def run_on_js(filename, passes, js_engine, source_map=False, extra_info=None, ju
     if cleanup:
       passes = [p for p in passes if p != 'cleanup'] # we will do it manually
 
-    split_memory = 'splitMemory' in passes
-
   if not minify_globals:
     with ToolchainProfiler.profile_block('js_optimizer.no_minify_globals'):
       pre = js[:start_funcs + len(start_funcs_marker)]
@@ -456,7 +454,8 @@ EMSCRIPTEN_FUNCS();
     for filename in filenames: temp_files.note(filename)
 
   with ToolchainProfiler.profile_block('split_closure_cleanup'):
-    if closure or cleanup or split_memory:
+    wasm_pthreads_memory_growth = shared.Settings.WASM and shared.Settings.USE_PTHREADS and shared.Settings.ALLOW_MEMORY_GROWTH
+    if closure or cleanup or wasm_pthreads_memory_growth:
       # run on the shell code, everything but what we js-optimize
       start_asm = '// EMSCRIPTEN_START_ASM\n'
       end_asm = '// EMSCRIPTEN_END_ASM\n'
@@ -470,10 +469,15 @@ EMSCRIPTEN_FUNCS();
           f.write(cl_sep)
           f.write(post_2)
         cld = cle
-        if split_memory:
-          if DEBUG: print('running splitMemory on shell code', file=sys.stderr)
-          cld = run_on_chunk(js_engine + [JS_OPTIMIZER, cld, 'splitMemoryShell'])
-          with open(cld, 'a') as f:
+        if wasm_pthreads_memory_growth:
+          if DEBUG: print('supporting wasm memory growth with pthreads', file=sys.stderr)
+          cld = run_on_chunk(js_engine + [JS_OPTIMIZER, cld, 'growableHeap'])
+          with open(cld, 'r') as f:
+            src = f.read()
+          with open(cld, 'w') as f:
+            with open(shared.path_from_root('src', 'growableHeap.js')) as g:
+              f.write(g.read() + '\n')
+            f.write(src)
             f.write(suffix_marker)
         if closure:
           if DEBUG: print('running closure on shell code', file=sys.stderr)
